/*
 * | Licensed Materials - Use restricted, please refer to the "Samples" terms
 * | and conditions in the IBM International Program License Agreement (IPLA).  
* (C) Copyright IBM Corporation. 1997, 2006. All Rights Reserved.
 */
/*	
* This program is an experimental approach to determining the 
* metrical structure of Latin epic verse by computer.
* Metrical analysis of verse is called "scansion."
*
* STRUCTURE OF EPIC VERSE
*
* Unlike English verse, whose metrical structure depends on 
* where the accent falls in individual words, Latin verse
* depends whether each syllable in each word is long or
* short.
*
* In epic verse, the basic pattern of long (_) and short(u)
* syllables is six feet, arranged as follows:
*
*	_ uu | _ uu | _ uu | _ uu | _ uu | - x  
*
* A long takes twice as long as a short to pronounce, 
* so the verse has, in effect, a 4/4 rhythm (in musical terms)
* with the long syllables equivalent to half notes, and the
* short syllables equivalent to quarter notes.
*
* The final syllable can be either long or short, so is
* represented by an "x".
*
* In all feet other than the 5th, you can substitute a long
* for the two shorts.
*
* SYLLABLE LENGTH
*
* Syllables can be long by position or by nature:
*
*	Syllables are long by position if they are followed
*	by two consonants, either in the same word or the
*	next. This is something the computer can easily 
*	detect.	
*
* 	Syllables are long by nature if they contain a
*	long vowel. Diphthongs (e.g., ae, au) are long,
*	which the computer can detect. Syllables are also
*	long if the simple vowel (a, e, i, o, u, or y) is
*	long--and this is something that a native speaker
*	would know, but that a computer (or beginning student)
*	cannot detect without access to a complete dictionary
*	of the language and a grammar book with all possible
*	noun and verb forms.
*
* HOW THIS PROGRAM DETERMINES SYLLABLE LENGTH
*
* This program marks as long syllables that are long by position
* and and syllables that contain diphthongs. It marks all other
* syllables as dubious (?).
*
* It then uses various "back-door" techniques to try to 
* find the length of the other syllables. For example,
* if it discovers a pattern _ ? _, it converts it to _ _ _,
* because _ u _ is impossible -- you never have a single
* short standing alone. It also counts the number of long
* and dubious syllables. Certain combinations make it possible
* to draw conclusions about the remainder of the verse.
*
* LIMITATIONS
*
* Language is notoriously irregular, and Latin verse has many 
* exceptions to the rules about syllable length. 
*
* This program includes some functions that deal with the
* exceptions (such as the the omission of the last vowel
* of a word when the next word begins with a vowel, which
* is called "elision"). But the program does not handle
* all irregularities.
*
* The program was designed using metrical material from 
* the introduction to Vergil's Aeneid. It was these seven
* verses--which as it turns out are quite regular--that 
* were used to structure the program and its irregularity-handling
* functions. There are many more irregularities than this program
* can handle, so a function was added to report when the metrical 
* analysis for any given line was unsuccessful. The program 
* can thus be used as a development tool used to refine the 
* program itself, and possibly to discover new statistical 
* information about the existing body of Latin epic verse.
* 
* The program currently reports two incorrectly scanned verses
* as successes. Further refinement is necessary to weed out
* the conditions that produce these incorrect "true's."  
*
*/
  
#include <stdio.h>

struct scanned_verse
{
	char verse[100];
	char work_copy[100];
	char letter_id[100];
	char scansion[100];
	char footmarker[100];
};

/* This structure includes:
* 	an array to hold the input verse;
*	an array for a working copy of the verse, to be modified
*		for elisions, punctuation, and double spaces;
*	an array to identify each letter of the verse as vowels,
*		diphthongs, and various types of consonants;
*	an array for the long, short, and dubious syllable
*		markings for the scansion;
*	and an array for vertical bars marking the vowel at which
*		each of the line's six feet begin.
*/

struct scanned_verse specimen;


void main()

{
	int i, check, j=0, k=0, result ;
	int check_scansion();
	FILE *fd = fopen("latinsample.txt","r");
	if (fd < 0) {
	    printf("Failed to open latinsample.txt file.\n");
	    return;
	}

	while(1)
	{
		fgets(specimen.verse, 100, fd);

		if(specimen.verse[0]=='0')/* Exit routine, which
					   * also summarizes the
					   * statistics for the run
					   * for successes and failures.
					   */
		{
			printf("\n%d successes, %d failures\n",j,k);
			if(j>0||k>0) /*To avoid core dump*/
			{
				result=100*(j)/(j+k);
				printf("\nSuccess rate: %d%%\n\n",result);
			}
			break;
		}


			/*The following code initializes the 
			 * string arrays of the structure.
			 */			
		strcpy(specimen.work_copy, specimen.verse);
		for (i=0;i<strlen(specimen.verse);i++)
		{
			specimen.letter_id[i]=' ';
			specimen.scansion[i]=' ';
			specimen.footmarker[i]=' ';
		}
		specimen.letter_id[i]='\0';
		specimen.scansion[i]='\0';
		specimen.footmarker[i]='\0';
 
			/* The following functions scan the verse */
		make_work_copy();
		mark_vowels();
		mark_cons();
		adj_qu();
		convert_semivowels();
		find_dips();
		find_elisions();
		mark_position();
		mark_1stlong();
		mark_last2feet();
		mark_finalvowels();
		mark_inf_imp();
		mark_os_as();
		mark_a_e();
		mark_regina();
		mark_dubiousvowels();
		adjust_singles();
		find_pairs();
		mark_feet();
		check=check_scansion();

			/* The following prints out successful scansions;
			 * admits failure for unsuccessful scansions;
			 * and increments the appropriate counter.
			 */
		if(check==1)
		{
			j++;
			printf("\n%s\n",specimen.footmarker);
			printf("%s\n",specimen.scansion);
			printf("%s\n\n\n",specimen.verse);
		}
		else
		{
			k++;
			printf("\n%s   FAILURE!\n",specimen.footmarker);
			printf("%s\n",specimen.scansion);
			printf("%s\n",specimen.verse);
			printf("%s\n",specimen.work_copy);
			printf("%s\n\n\n",specimen.letter_id);
		}
	}
}

make_work_copy()
/*  This function converts the verse to lower case, removes silent
 *  h's, and removes punctuation, in order to
 *  make a working copy that is the basis for the scansion.
 */
{
	int i;

	for(i=0;i<strlen(specimen.verse);i++)
	{	
		specimen.work_copy[i]=tolower(specimen.work_copy[i]);
	}

	for(i=0;i<strlen(specimen.verse);i++)
	{
		if ((specimen.verse[i]=='h'&&specimen.verse[i-1]!='i')||
			specimen.verse[i]=='.'||
			specimen.verse[i]==','||
			specimen.verse[i]==';'||
			specimen.verse[i]==':'||
			specimen.verse[i]=='?')

			{
				specimen.work_copy[i]=' ';	
			}
	}
}

mark_vowels()
/*This function marks vowels in the letter_id array.*/
{
	int i;	
	for (i=0;i<strlen(specimen.verse);i++)
	{
		if (specimen.work_copy[i]=='a'||
		    specimen.work_copy[i]=='e'||
		    specimen.work_copy[i]=='i'||
		    specimen.work_copy[i]=='o'||
		    specimen.work_copy[i]=='u'||
		    specimen.work_copy[i]=='y')
		{
			specimen.letter_id[i]='v';
		}
	}	
	
}

mark_cons()
/*This function marks the letters p, b, and t as "m" (for "mutes,"
 *which have a special metrical significance, to be explained later),
 *the letters l and r as "l" (for "liquids," again significant),
 *the letter x as "x" (it is equivalent to k+s, and so a double
 *consonant), and the remaining Latin consonants as "c."
 */

{
	int i;	
	for (i=0;i<strlen(specimen.verse);i++)
	{
		if (specimen.work_copy[i]=='c'||
		    specimen.work_copy[i]=='d'||
		    specimen.work_copy[i]=='f'||
		    specimen.work_copy[i]=='g'||
		    specimen.work_copy[i]=='j'||
		    specimen.work_copy[i]=='k'||
		    specimen.work_copy[i]=='m'||
		    specimen.work_copy[i]=='n'||
		    specimen.work_copy[i]=='q'||
		    specimen.work_copy[i]=='s'||
		    specimen.work_copy[i]=='v')
		{
			specimen.letter_id[i]='c';
		}
		else if (specimen.work_copy[i]=='p'||
		    specimen.work_copy[i]=='b'||
		    specimen.work_copy[i]=='t')
		{
			specimen.letter_id[i]='m';
		}
		else if (specimen.work_copy[i]=='r'||
		    specimen.work_copy[i]=='l')
		{
			specimen.letter_id[i]='l';
		}
		else if (specimen.work_copy[i]=='x')
		{
			specimen.letter_id[i]='x';
		}
	}	
	
}
adj_qu()
/*This function converts the string "qu" to a single consonant,
 * which is how it is treated metrically, in the letter_id array.
 */

{
	int i;
	for (i=0;i<strlen(specimen.verse);i++)
	{
		if(specimen.work_copy[i]=='u' && specimen.work_copy[i-1]=='q')
		{
			specimen.letter_id[i]=' ';
		}
	}
}

convert_semivowels()
/* This function converts i, when found in certain positions, to
 *  a consonantal j. It also removes the u from the root suad
 * (persuade).
 */
{
	int i;
	for (i=0;i<strlen(specimen.verse);i++)
	{
		if(specimen.work_copy[i]=='i' &&
			 specimen.letter_id[i-1]!='c'&&
			 specimen.letter_id[i-1]!='m'&&
			 specimen.letter_id[i-1]!='l'&&
			 specimen.verse[i-2]!='q'&&
			(specimen.work_copy[i+1]=='a'||
			 specimen.work_copy[i+1]=='e'||
			 specimen.work_copy[i+1]=='o'||
			 specimen.work_copy[i+1]=='u'))
 
		{
			specimen.work_copy[i]='j';
			specimen.letter_id[i]='c';
		}
		if(specimen.work_copy[i]=='u' &&
			 specimen.work_copy[i-1]=='s'&&
			 specimen.work_copy[i+1]=='a'&&
			 specimen.work_copy[i+2]=='d')
 
		{
			specimen.letter_id[i]=' ';
		}
	}
}

find_dips()
/* This function marks diphthongs in the letter_id array
 * and marks the diphthong long in the scansion array.
 */

{
	int i;
	for (i=0;i<strlen(specimen.verse);i++)
	{
		if(specimen.work_copy[i]=='a' &&
			(specimen.work_copy[i+1]=='e'||
			 specimen.work_copy[i+1]=='u'))
		{
			specimen.letter_id[i]='d';
			specimen.letter_id[i+1]=' ';
			specimen.scansion[i]='_';
		}
		else if(specimen.work_copy[i]=='e' &&
			(specimen.work_copy[i+1]=='u'&&
			 specimen.work_copy[i-1]!='d'&&
			 specimen.work_copy[i-1]!='m'))
		{
			specimen.letter_id[i]='d';
			specimen.letter_id[i+1]=' ';
			specimen.scansion[i]='_';
		}
		else if(specimen.work_copy[i]=='o' &&
			(specimen.work_copy[i+1]=='e'))
		{
			specimen.letter_id[i]='d';
			specimen.letter_id[i+1]=' ';
			specimen.scansion[i]='_';
		}
	}
}

find_elisions()
/* This function detects locations where elision occurs,
 * and removes the elided letters from the working copy.
 * An elision occurs when a word ending in a vowel or a vowel+m
 * is followed immediately by a word beginning with a vowel.
 * For example, in the phrase "multum ille" the "um" is elided,
 * or dropped--not pronounced at all.
 */
{
	int i;
	for (i=0;i<strlen(specimen.verse);i++)
	{
		if(specimen.work_copy[i]=='m' &&
			specimen.work_copy[i+1]==' '&&
			(specimen.work_copy[i+2]=='a'||
			specimen.work_copy[i+2]=='e'||
			specimen.work_copy[i+2]=='i'||
			specimen.work_copy[i+2]=='o'||
			specimen.work_copy[i+2]=='u'))
		{
			specimen.letter_id[i-1]=' ';
			specimen.letter_id[i+1]=' ';
			specimen.work_copy[i]=' ';
			specimen.work_copy[i-1]=' ';
		}
		else if((specimen.work_copy[i]=='a'||
			specimen.work_copy[i]=='e'|| 
			specimen.work_copy[i]=='i'|| 
			specimen.work_copy[i]=='o'|| 
			specimen.work_copy[i]=='u')&& 
			specimen.work_copy[i+1]==' '&&
			(specimen.work_copy[i+2]=='a'||
			specimen.work_copy[i+2]=='e'||
			specimen.work_copy[i+2]=='i'||
			specimen.work_copy[i+2]=='o'||
			specimen.work_copy[i+2]=='u'))
		{
			specimen.letter_id[i]=' ';
			specimen.work_copy[i]=' ';
		}
		else if((specimen.work_copy[i]=='a'||
			specimen.work_copy[i]=='e'|| 
			specimen.work_copy[i]=='i'|| 
			specimen.work_copy[i]=='o'|| 
			specimen.work_copy[i]=='u'||
			specimen.letter_id[i]=='d')&&
			specimen.work_copy[i+1]==' '&&
			specimen.work_copy[i+2]==' '&&
			(specimen.work_copy[i+3]=='a'||
			specimen.work_copy[i+3]=='e'||
			specimen.work_copy[i+3]=='i'||
			specimen.work_copy[i+3]=='o'||
			specimen.work_copy[i+3]=='u'))
		{
			specimen.letter_id[i]=' ';
			specimen.work_copy[i]=' ';
		}
		else if((specimen.work_copy[i]=='a'||
			specimen.work_copy[i]=='e'|| 
			specimen.work_copy[i]=='i'|| 
			specimen.work_copy[i]=='o'|| 
			specimen.work_copy[i]=='u'||
			specimen.letter_id[i]=='d')&&
			specimen.work_copy[i+1]==' '&&
			specimen.work_copy[i+2]==' '&&
			specimen.work_copy[i+3]==' '&&
			(specimen.work_copy[i+4]=='a'||
			specimen.work_copy[i+4]=='e'||
			specimen.work_copy[i+4]=='i'||
			specimen.work_copy[i+4]=='o'||
			specimen.work_copy[i+4]=='u'))
		{
			specimen.letter_id[i]=' ';
			specimen.work_copy[i]=' ';
		}
		else if (specimen.letter_id[i]=='d'&&
			 specimen.work_copy[i+1]==' '&&
			 specimen.work_copy[i+2]==' '&&
			 specimen.work_copy[i+3]==' '&&
			 (specimen.work_copy[i+3]=='a'||
			 specimen.work_copy[i+3]=='e'||
			 specimen.work_copy[i+3]=='i'||
			 specimen.work_copy[i+3]=='o'||
			 specimen.work_copy[i+3]=='u'))
		{
			specimen.letter_id[i]=' ';
			specimen.work_copy[i]=' ';
		}
	}
}

mark_position()
/* This function marks vowels that are long by position--followed
 * by two consonants. The combination of mute+liquid, however
 * (for example, tr) does not always make position, and so
 * is not marked here.
 */
 
{

	int i;
	for (i=0;i<strlen(specimen.verse);i++)
	{
		if(specimen.letter_id[i]=='v'&&
			((specimen.letter_id[i+1]=='c'&& 
			specimen.letter_id[i+2]=='c')|| 
			(specimen.letter_id[i+1]=='c'&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]=='c')|| 
			(specimen.letter_id[i+1]=='c'&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]==' '&& 
			specimen.letter_id[i+4]=='c')|| 

			(specimen.letter_id[i+1]==' '&& 
			specimen.letter_id[i+2]=='c'&& 
			specimen.letter_id[i+3]=='c')|| 
			(specimen.letter_id[i+1]==' '&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]=='c'&& 
			specimen.letter_id[i+4]=='c'))) 

		{
			specimen.scansion[i]='_';
		}
	}

	for (i=0;i<strlen(specimen.verse);i++)
	{
		if(specimen.letter_id[i]=='v'&&
			((specimen.letter_id[i+1]=='l'&& 
			specimen.letter_id[i+2]=='l')|| 
			(specimen.letter_id[i+1]=='l'&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]=='l')|| 
			(specimen.letter_id[i+1]=='l'&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]==' '&& 
			specimen.letter_id[i+4]=='l')|| 

			(specimen.letter_id[i+1]==' '&& 
			specimen.letter_id[i+2]=='l'&& 
			specimen.letter_id[i+3]=='l')|| 
			(specimen.letter_id[i+1]==' '&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]=='l'&& 
			specimen.letter_id[i+4]=='l'))) 

		{
			specimen.scansion[i]='_';
		}
	}
	for (i=0;i<strlen(specimen.verse);i++)
	{
		if(specimen.letter_id[i]=='v'&&
			((specimen.letter_id[i+1]=='m'&& 
			specimen.letter_id[i+2]=='m')|| 
			(specimen.letter_id[i+1]=='m'&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]=='m')|| 
			(specimen.letter_id[i+1]=='m'&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]==' '&& 
			specimen.letter_id[i+4]=='m')|| 

			(specimen.letter_id[i+1]==' '&& 
			specimen.letter_id[i+2]=='m'&& 
			specimen.letter_id[i+3]=='m')|| 
			(specimen.letter_id[i+1]==' '&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]=='m'&& 
			specimen.letter_id[i+4]=='m'))) 

		{
			specimen.scansion[i]='_';
		}
	}

	for (i=0;i<strlen(specimen.verse);i++)
	{
		if(specimen.letter_id[i]=='v'&&
			((specimen.letter_id[i+1]=='c'&& 
			specimen.letter_id[i+2]=='m')|| 
			(specimen.letter_id[i+1]=='c'&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]=='m')|| 
			(specimen.letter_id[i+1]=='c'&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]==' '&& 
			specimen.letter_id[i+4]=='m')|| 

			(specimen.letter_id[i+1]==' '&& 
			specimen.letter_id[i+2]=='c'&& 
			specimen.letter_id[i+3]=='m')|| 
			(specimen.letter_id[i+1]==' '&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]=='c'&& 
			specimen.letter_id[i+4]=='m'))) 

		{
			specimen.scansion[i]='_';
		}
	}

	for (i=0;i<strlen(specimen.verse);i++)
	{
		if(specimen.letter_id[i]=='v'&&
			((specimen.letter_id[i+1]=='m'&& 
			specimen.letter_id[i+2]=='c')|| 
			(specimen.letter_id[i+1]=='m'&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]=='c')|| 
			(specimen.letter_id[i+1]=='m'&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]==' '&& 
			specimen.letter_id[i+4]=='c')|| 

			(specimen.letter_id[i+1]==' '&& 
			specimen.letter_id[i+2]=='m'&& 
			specimen.letter_id[i+3]=='c')|| 
			(specimen.letter_id[i+1]==' '&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]=='m'&& 
			specimen.letter_id[i+4]=='c'))) 

		{
			specimen.scansion[i]='_';
		}
	} 

	for (i=0;i<strlen(specimen.verse);i++)
	{
		if(specimen.letter_id[i]=='v'&&
			((specimen.letter_id[i+1]=='c'&& 
			specimen.letter_id[i+2]=='l')|| 
			(specimen.letter_id[i+1]=='c'&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]=='l')|| 
			(specimen.letter_id[i+1]=='c'&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]==' '&& 
			specimen.letter_id[i+4]=='l')|| 

			(specimen.letter_id[i+1]==' '&& 
			specimen.letter_id[i+2]=='c'&& 
			specimen.letter_id[i+3]=='l')|| 
			(specimen.letter_id[i+1]==' '&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]=='c'&& 
			specimen.letter_id[i+4]=='l'))) 

		{
			specimen.scansion[i]='_';
		}
	}

	for (i=0;i<strlen(specimen.verse);i++)
	{
		if(specimen.letter_id[i]=='v'&&
			((specimen.letter_id[i+1]=='l'&& 
			specimen.letter_id[i+2]=='c')|| 
			(specimen.letter_id[i+1]=='l'&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]=='c')|| 
			(specimen.letter_id[i+1]=='l'&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]==' '&& 
			specimen.letter_id[i+4]=='c')|| 

			(specimen.letter_id[i+1]==' '&& 
			specimen.letter_id[i+2]=='l'&& 
			specimen.letter_id[i+3]=='c')|| 
			(specimen.letter_id[i+1]==' '&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]=='l'&& 
			specimen.letter_id[i+4]=='c'))) 

		{
			specimen.scansion[i]='_';
		}
	}
	for (i=0;i<strlen(specimen.verse);i++)
	{
		if(specimen.letter_id[i]=='v'&&
			((specimen.letter_id[i+1]=='l'&& 
			specimen.letter_id[i+2]=='m')|| 
			(specimen.letter_id[i+1]=='l'&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]=='m')|| 
			(specimen.letter_id[i+1]=='l'&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]==' '&& 
			specimen.letter_id[i+4]=='m')|| 

			(specimen.letter_id[i+1]==' '&& 
			specimen.letter_id[i+2]=='l'&& 
			specimen.letter_id[i+3]=='m')|| 
			(specimen.letter_id[i+1]==' '&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]=='l'&& 
			specimen.letter_id[i+4]=='m'))) 

		{
			specimen.scansion[i]='_';
		}
	}

	for (i=0;i<strlen(specimen.verse);i++)
	{
		if(specimen.letter_id[i]=='v'&&
			((specimen.letter_id[i+1]=='x')|| 
			(specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]=='x')|| 
			(specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]==' '&& 
			specimen.letter_id[i+4]=='x')|| 

			(specimen.letter_id[i+1]==' '&& 
			specimen.letter_id[i+2]=='x')|| 
			(specimen.letter_id[i+1]==' '&& 
			specimen.letter_id[i+2]==' '&& 
			specimen.letter_id[i+3]=='x'))) 

		{
			specimen.scansion[i]='_';
		}
	}

}
	
mark_last2feet()
/* This function marks the last two feet, which (in classical
 * epic meter, at least) always have the structure _ u u | _ x,
 * where the x can be either long or short.
 */

{

	int i=strlen(specimen.verse), j=0;

	while(j<1)
	{
		if(specimen.letter_id[i]=='v'||
			specimen.letter_id[i]=='d')
		{
			specimen.scansion[i]='x';	
			j++;
		}
		i--;
	}
		
	while(j>=1 && j<2 && i>0)
	{
		if(specimen.letter_id[i]=='v'||
			specimen.letter_id[i]=='d')
		{
			specimen.scansion[i]='_';
			specimen.footmarker[i]='|';
			j++;
		}
		i--;
	}

	while(j>=2 && j<4 && i>0)
	{
		if(specimen.letter_id[i]=='v')
		{
			specimen.scansion[i]='u';
			j++;
		}
		i--;
	}

	while(j>=4 && j<5 && i>0)
	{
		if(specimen.letter_id[i]=='v'||
			specimen.letter_id[i]=='d')
		{
			specimen.scansion[i]='_';
			specimen.footmarker[i]='|';
			j++;
		}
		i--;
	}

}

mark_1stlong()
/* This function marks the first foot, which is ALWAYS long. */
 
{
	int i=0, j;
	for (j=0;j<=1;j++)
	{
		for(i=0;i<4;i++)
		{	
			if(specimen.letter_id[i]=='v')
			{
				specimen.scansion[i]='_';
				break;
			}
		}
	}
}

mark_finalvowels()
/* This function marks i, o, or u as long when they are the
 * last letter of a word. It also marks the e as long in the
 * common words me, te, and se (me, you, and himself).
 * It excludes u, if the preceding letter is q and the following
 * vowel is elided, as well as other vowels when they are
 * in the final position because of elision. Note that this
 * process is carried out only for the first four feet, so
 * that the final syllable (marked x in the scansion array)
 * is not overwritten.
 */
 
{
	int i, j;
	for(i=0;i<strlen(specimen.verse);i++)
	{
		if(specimen.footmarker[i+1]=='|')
		{
			break;
		}
	}
	for(j=0;j<i;j++)
	{
		if(((specimen.work_copy[j]=='i'||
			specimen.work_copy[j]=='o'||
			specimen.work_copy[j]=='u')&&
			specimen.work_copy[j+1]==' '&&
			specimen.work_copy[j-i]!='q'&&
			(specimen.verse[j+1]==' '||
			 specimen.verse[j+1]==','||
			 specimen.verse[j+1]=='.'||
			 specimen.verse[j+1]==';'||
			 specimen.verse[j+1]=='?'))||
			(specimen.work_copy[j]=='e'&&
			(specimen.work_copy[j-1]=='m'||
			specimen.work_copy[j-1]=='s'||
			specimen.work_copy[j-1]=='t')&&
			specimen.work_copy[j-2]==' '&&
			specimen.work_copy[j+1]==' '))

		{
			specimen.scansion[j]='_';
		}
	}
}

mark_inf_imp()
/* This function marks a and i long in the common endings
 * are, ate, and ire. No worry about overwriting the "x" here.
 * (These are infinitives and imperatives.)
 */

{
	int i;	
	for (i=0;i<strlen(specimen.verse);i++)
	{
		if((specimen.work_copy[i]=='a'&&
			(specimen.work_copy[i+1]=='t'|| 
			specimen.work_copy[i+1]=='r')&& 
			specimen.work_copy[i+2]=='e'&& 
			specimen.work_copy[i+3]==' ')&&
			(specimen.work_copy[i-1]!='m'&&
			specimen.work_copy[i-2]!=' ')||
			 
			(specimen.work_copy[i]=='i'&&
			specimen.work_copy[i+1]=='r'&& 
			specimen.work_copy[i+2]=='e'&& 
			specimen.work_copy[i+3]==' '))
		{
			specimen.scansion[i]='_';
		}
	}
}
			
mark_os_as()
/*This function marks accusative plurals in os and as, which
 * are always long.
 */

{
	int i;	
	for (i=0;i<strlen(specimen.verse);i++)
	{
		if((specimen.work_copy[i]=='a'||
			specimen.work_copy[i]=='o')&&
			specimen.scansion[i]!='x'&&
			(specimen.work_copy[i+1]=='s'&&
			specimen.work_copy[i+2]==' ')) 
			 
		{
			specimen.scansion[i]='_';
		}
	}
}

mark_a_e()
/* This function marks the prepositions a and e, which are
 * always long.
 */

{
	int i;	
	for (i=0;i<strlen(specimen.verse);i++)
	{
		if((specimen.work_copy[i]=='a'||
			specimen.work_copy[i]=='e')&&
			(specimen.work_copy[i+1]==' '&&
			specimen.work_copy[i-1]==' ')) 
			 
		{
			specimen.scansion[i]='_';
		}
	}
}

mark_regina()
/*This function marks the e and i in the commonly occurring
 * word regina (queen) as long.
 */
{
	int i;	
	for (i=0;i<strlen(specimen.verse);i++)
	{
		if(specimen.work_copy[i]=='r'&&
			specimen.work_copy[i+1]=='e'&&
			specimen.work_copy[i+2]=='g'&&
			specimen.work_copy[i+3]=='i'&&
			specimen.work_copy[i+4]=='n')
			 
		{
			specimen.scansion[i+1]='_';
			specimen.scansion[i+3]='_';
		}
	}
}

mark_dubiousvowels()
/* This function marks vowels not already marked long in the
 * scansion array as "?" in the scansion array. They are
 * indeterminate at this point.
 */
{
	int i;
	for (i=0;i<strlen(specimen.verse);i++)
	{
		if(specimen.letter_id[i]=='v'
			&& specimen.scansion[i]==' ')

		{
			specimen.scansion[i]='?';
		}
	}
}

adjust_singles()
/* Any single indeterminate vowels--indeterminate vowels surrounded
 * by longs--are marked long in this function. The pattern _ u _
 * never occurs.
 */

{
int i=0, j;
marker:
while(i<strlen(specimen.scansion))
{
	if(specimen.scansion[i]==' '||specimen.scansion[i]=='?')
	{
		i++;
	}
	else
	{
		i++;
		while(i<strlen(specimen.verse))
		{
			if(specimen.scansion[i]==' ')
			{
				i++;
			}	
			else if(specimen.scansion[i]=='_')
			{
				goto marker;
			}
			else if(specimen.scansion[i]=='u')
			{
				i++;
				goto marker;
			}
			else
			{
				j=i;
				i++;

				while(i<strlen(specimen.verse))
				{
					if(specimen.scansion[i]==' ')
					{
						i++;
					}
					else if (specimen.scansion[i]=='?')
					{
						i++;
						goto marker;
					}
					else
					{
						specimen.scansion[j]='_';
						goto marker;
					}
				}
					
			}
		}
	}
}
}

find_pairs()
/* This function counts the marked longs and dubious vowels, and
 * converts pairs of dubious syllables to pairs of short vowels under
 * specific mathematical conditions. 
 */

{
	int i,j=0,k=0;
	for(i=0;i<strlen(specimen.verse);i++)
	{
		if(specimen.scansion[i]=='_')
		{
			j++;
		}
		else if(specimen.scansion[i]=='?')
		{
			k++;
		}
	}
	/*printf("%d, %d",k,j); Diagnostic output */
	if(k==2 || (k==4 && j==8) || (k==6 && j==7))
	{
		for(i=0;i<strlen(specimen.verse);i++)
		{
			if(specimen.scansion[i]=='?')
			{
				specimen.scansion[i]='u';
			}
		}
	}
}
	
mark_feet()
/* This function marks the beginning of the feet in the
 * verse. It occasionally errs--it needs refinement so that
 * it looks at the total number of marked and dubious syllables.
 * It can probably be used to determine the lengths of some
 * of the dubious syllables, but that's for later.
 */
{
	int i=0,j;
	marker:
	while(i<strlen(specimen.verse))
	{
		if(specimen.scansion[i]==' ')
		{
			i++;
		}
		else if(specimen.scansion[i]=='_')
		{
			j=i;
			i++;
			while(i<strlen(specimen.verse))
			{
				if(specimen.scansion[i]=='u')
				{
					specimen.footmarker[j]='|';
					i++;
					goto marker;
				}
				else if(specimen.scansion[i]=='_')
				{
					specimen.footmarker[j]='|';
					i++;
					goto marker;
				}
				else
				{
					i++;
				}
			}
		}
		else
		{
			i++;
		}
	}
}

int check_scansion()
/* This function makes a primitive check of correctness. 
 * If checks that there are no dubious syllables remaining,
 * and checks that there are six foot markers (|) in the
 * scansion array. If so, it chalks the verse up as OK; if
 * not, it condemns it. This information is passed back
 * to main, where it controls the output and contributes
 * to the success statistics.
 */
 
{
	int i, j=0, k=0;
	int check;
	for (i=0;i<strlen(specimen.verse);i++)
	{
		if(specimen.scansion[i]=='?')
		{
			j++;
		}
		if(specimen.footmarker[i]=='|')
		{
			k++;
		}
	}
	if(j>0||k!=6)
	{
		check=0;
		return (check);
	}
	else
	{
		check=1;
		return (check);
	}
}
